------------------------------------------------------------------------------------------
-- Whammy
-- Created By:  Jaycyn of Hyjal
--
-- What is it: Heal, Decurse and more with a point and click
-- License   : Distribute as you wish, please give credit to the author, please send
--           :   bugs and suggestions instead of modifying code yourself.
------------------------------------------------------------------------------------------

local WhammyVers = "0.3.6";
local WhammyDate = "1/30/2007";
local W_kHealthBarWidth = 70; --the width of the health bar on each unit button

-- below are functions dealing with clicking in the unit buttons to cast spells
--   note that the default action for each button is assigned when the main frame loads.
function W_UnitButton_Onload()
	--W_ShowMessage("Unit_Button_Onload");
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp"); -- the buttons need to see the mouse clicks
	-- this refers to the frame/button that called this event
end



--
-- Handle Unit Button Mouse actions
--
function W_UnitButton_MouseDown() --MouseDown and up just highlight the unit name when clicked. arg1 is the LeftButton, etc. unused
	local buttonName = this:GetName(); --get the name of the unit button that triggered this event
	getglobal(buttonName.."_UnitName"):SetFontObject( W_kFontNameHighlight );
end

function W_UnitButton_MouseUp() -- unhighlight the name
	local buttonName = this:GetName(); --get the button
	local unitName = W_Get_Unit_Name_From_Button( buttonName ); --get the name of the player the button is assigned to
	local playerName = UnitName("player"); --get the name of this player
	
	--W_ShowMessage("Unit Button Mouse Up.  Unit="..unitName.."  player="..playerName);
	
	if ( playerName == unitName ) then -- if this is the player button, reset it's font back to the player style text
		getglobal(buttonName.."_UnitName"):SetFontObject( W_kFontNamePlayer );
	else --otherwise it's someone elses button, so reset it to the others color
		getglobal(buttonName.."_UnitName"):SetFontObject( W_kFontNameOthers );
	end
end
-- End Handle Unit Button Mouse actions



--
-- below are items dealing with the main Whammy Frame: registering for events, mouse clicks etc
--
function W_Frame_Onload(self)
	-- W_ShowMessage("Frame_Onload");

	-- Register the game events neccesary for our functionality
	self:RegisterEvent("ADDON_LOADED");
	--self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("PLAYER_AURAS_CHANGED");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("PLAYER_REGEN_DISABLED"); --use this to know if we enter combat so we can disable the drop downs
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	--self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	--					CHAT_MSG_SPELL_AURA_GONE_SELF
	
	-- set up the slash commands
	SlashCmdList["WHAMMY"] = W_Slash_Command_Handler;
	SLASH_WHAMMY1 = "/whammy";
	SLASH_WHAMMY2 = "/wham";
end

-- Handle stuff relevent to the Whammy Frame
function W_Frame_MouseDown(self) --this toggles the config frame
	--W_ShowMessage("Frame_MouseDown");
	
	if arg1 == "LeftButton" then -- drag the frame
		self:StartMoving();
	elseif arg1 == "RightButton" then --need to show a frame that lets the user turn mouseovers on or off
		W_Config_Frame_Toggle();
	end
end

function W_Toggle_Config_Frame(self)
	if arg1 == "RightButton" then --need to show a frame that lets the user turn mouseovers on or off
		local f = UIParent;
		local w = f:GetWidth();
		W_ShowMessage("width is: "..w)
	end
	
end

-- Based upon our registered events, we want different things to happen when each event occurs.
function W_Frame_OnEvent(self, event, ...)
	--W_ShowMessage( event );
	
	if ( event == "ADDON_LOADED" and arg1 == "Whammy") then -- the event occurs after the saved vars are loaded
		W_ShowMessage("Version: "..WhammyVers.." "..WhammyDate.." loaded"); --" V0.21a 1/6/2007 Loaded");
		this:UnregisterEvent("ADDON_LOADED");
		gWhammyIsLoaded = true;
		W_Handle_Addon_Loaded_Event(); --set up button frames, etc
	end
	
	if ( gWhammyIsLoaded == true ) then --only take action on these events if Whammy is loaded
		if ( event == "VARIABLES_LOADED" ) then --this event occurs after we load in any saved variables. Set drop downs to default values
			--W_ShowMessage("Variables Loaded Event");
			return;
		end
		
		if ( event =="PLAYER_LOGIN") then
			if ( GetNumRaidMembers() == 0 ) then --not a raid
				W_Handle_Party_Members_Changed_Event(); --set the frame to the appropriate size for either player or party
			else -- its a raid, set up the frame
				W_Handle_Raid_Roster_Update_Event();
			end
			return;
		end
		
		if ( event == "PLAYER_AURAS_CHANGED" ) then --watch for this player
			--W_Handle_Player_Auras_Changed_Event(); --this will call Handle Unit Aura and pass it player
			W_Handle_Unit_Aura_Event("player");
			return;
		end
		
		if ( event == "UNIT_AURA" and arg1  ~= "player" and arg1 ~= "target" ) then --watch for any player in the party or raid, arg1 is the unitID
			if ( W_Is_Unit_Player_Party_Raid( arg1 ) ) then -- we only care about player, party or raid
				W_Handle_Unit_Aura_Event( arg1 ); --arg1 is the unitID of the unit whose buff changed
			end
			return;
		end
		
		if ( event == "PARTY_MEMBERS_CHANGED" ) then
			if ( GetNumRaidMembers() == 0 ) then --we only care about this event if we are not in a raid since that will
				W_Handle_Party_Members_Changed_Event(); --    generate an event as well
			end
			return;
		end
		
		if ( event == "RAID_ROSTER_UPDATE" ) then
			W_Handle_Raid_Roster_Update_Event();
			return;
		end
		
		if ( event == "UNIT_HEALTH") then
			if ( arg1 ~= "target" ) then --ignore it if we heal someone that we clicked on
				if ( GetNumRaidMembers() == 0 ) then -- handle health events for player or party
					W_Handle_Party_Unit_Health_Event( arg1 );
				else -- handle health events for someone in the raid group		
					local id, numberFound = gsub(arg1, "raid([0-9]+)", "%1");
					if ( numberFound == 1 ) then
						W_Handle_Raid_Unit_Health_Event(id);
					end
				end
				return
			end
		end
		
		if ( event == "PLAYER_REGEN_DISABLED" ) then --we have entered combat
			--W_ShowMessage("Entered Combat");
			W_Handle_Combat( true );
		end
	
		if ( event == "PLAYER_REGEN_ENABLED" ) then --we have entered combat
			--W_ShowMessage("Left Combat");
			W_Handle_Combat( false );
		end
		
--		if ( event == "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS" ) then
--			W_ShowMessage("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
--		end
	end
end



--
--------  EVENT HANDLERS --------

-- Handle changes to the party or raid
function W_Handle_Addon_Loaded_Event()
	if (W_SavedSpells == nil) then --if no saved spells were loaded
		W_Set_Saved_Spells(); --will set spell table defaults
	end
	
	W_Add_Drop_Down_Options();
	
	if ( W_gHealthLevel == nil ) then
		W_gHealthLevel = "Off"; --off by default
	end
	
	W_Create_Unit_Buttons(); --create all of the unit buttons
end

-- called after the spell tables have either been loaded or defaulted, to add any addtional button options: Trade, Whisper etc
function W_Add_Drop_Down_Options()
	local charType = UnitClass("player");
	local tableName = ("W_"..charType.."SpellTable"); --figure out which table to use based on char type
	local t = getglobal(tableName); -- a reference to the table
	table.insert(t, "Trade");
	table.insert(t, "Whisper");
end



-- called when the party members change (add, leave) - will only be called for solo and party events
function W_Handle_Party_Members_Changed_Event()
	--W_ShowMessage("Party members changed event");
	
	W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player
	W_Set_Unit_Button_Click("W_Unit_Button1"); --set the spells for this player

	local numberOfPartyMembers = GetNumPartyMembers(); -- is there anyone else in the party? 0-4 excluding the player

	if ( numberOfPartyMembers == 0 ) then -- the player is by themself, reset the other buttons and frame size
		for i = 1,4 do --1 through 4 are the other party members i.e. party1 is the first other party member etc
			W_ClearButton( "W_Unit_Button"..(i+1) ); --clear the name from the button
		end
		W_Set_Whammy_Frame_Size( W_kPlayer );
	else -- player is in a party, populate the other player buttons and set the frame size
		for i = 1,4 do --1 through 4 are the other party members i.e. party1 is the first other party member etc
			W_ClearButton( "W_Unit_Button"..(i+1) ); --clear the name, look etc for each button so it will be ready for the new player
			if UnitExists("party"..i) then
				W_Set_Unit_Button_Name( "W_Unit_Button"..(i+1), "party"..i );
				W_Set_Unit_Button_Click( "W_Unit_Button"..(i+1) );
			end
		end
		W_Set_Whammy_Frame_Size( W_kParty );
	end
end

--called when the raid roster changes or upon player login if they are in a raid
function W_Handle_Raid_Roster_Update_Event()
	local name, rank, subgroup, level, class, fileName, zone, online, isDead;
	local unitCount, pos;
	local unitButtonName, unitID;
	local subGroupUnitCount = {0,0,0,0,0,0,0,0}; --keep track of how many raid members are in each raid group. Starts at 0 (5 per group)
	W_gUnitButtonTable = {}; --clear out the global lookup table that links player unitID'S to the unit buttons they are in,
	
	--W_ShowMessage("Raid Roster Update")
	
	local numRaidMembers = GetNumRaidMembers();
	
	W_Clear_Raid_Buttons(); --roster changed so clear out all the raid buttons
	
	if ( numRaidMembers == 0 ) then -- if there are no raid members, reset size etc to just the player
		W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player
		W_Set_Unit_Button_Click("W_Unit_Button1"); --set the spells for this player
		W_Set_Whammy_Frame_Size( W_kPlayer ); --set the frame size to just player
	else
		W_Set_Whammy_Frame_Size( W_kRaid );
		for id = 1,numRaidMembers do --walk through all the raid members
			name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(id);	--get their name and subgroup
			subGroupUnitCount[ subgroup ] = subGroupUnitCount[ subgroup ] + 1; --keep track of how many units are in each subgroup
	
			unitCount = subGroupUnitCount[ subgroup ]; --get the next unit position based on the unit count of the subgroup
			
			pos = ( ( (subgroup-1) * 5) + unitCount ) -- first subgroup, first position will be 1, 8th subgroup, first position will be 36
			
			unitButtonName = "W_Unit_Button"..pos
			unitID = "raid"..id
			
			W_Set_Unit_Button_Name( unitButtonName, unitID );
			W_Set_Unit_Button_Click( unitButtonName );
			W_gUnitButtonTable[ unitID ] = unitButtonName; --add the buttons that are occupied to the table so we can look them up later
		end
	end
end

-- clear out all the raid buttons and reset them to default
function W_Clear_Raid_Buttons()
	for i = 1,W_kMaxUnitButtons do --clear out the raid buttons
		W_ClearButton( "W_Unit_Button"..i ); --clear the name from the button
	end
end



--
-- Handle Raid Unit health events and party or player health events
function W_Handle_Raid_Unit_Health_Event( id )
	local name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(id); --get the unit name and stuff
	local unitID = "raid"..id
	local unitButtonName = W_Get_Unit_Button_From_UnitID( unitID ); --look up the unit id via the global table and return which button it is in
	local unitButtonNameTextFrame = (unitButtonName.."_UnitName");
	
	--W_ShowMessage( unitID.."  is in unitbutton: "..unitButtonName );
	
	if ( online ) then
		if ( UnitIsUnit( unitID, "player") ) then --if they were offline and are now online, color the text correctly
			getglobal(unitButtonNameTextFrame):SetFontObject( W_kFontNamePlayer );
		else
			getglobal(unitButtonNameTextFrame):SetFontObject( W_kFontNameOthers );
		end
		
		if ( isDead ) then
			W_Set_Unit_Button_Texture( unitButtonName, W_kUnitDead ); --unit is dead, grey out the unit button texture
			W_Set_Health_Bar_Value( unitButtonName, 1); --set the healthbar to 1 (0 doesnt work)
		else --alive, update health bar
			local maxHealth = UnitHealthMax(unitID);
			local currentHealth = UnitHealth(unitID);
			local healthFraction = currentHealth / maxHealth;
			local healthBarAmount = W_kHealthBarWidth * healthFraction;
			
			W_Set_Unit_Button_Texture( unitButtonName, W_kUnitAlive);
			W_Set_Health_Bar_Value( unitButtonName, healthBarAmount );
			
			if W_gHealthLevel ~= "Off" then --if the user wants to be notified for low health, let them know
				--if the unit has low health, make the texture of the unit button red to get attention
				local healthTargetPercent = tonumber(W_gHealthLevel)/100; --health level is the % health selected in the drop down
				if ( healthFraction < healthTargetPercent ) then -- w_gHealthLevel is a saved var
					W_Set_Unit_Button_Texture( unitButtonName, W_kUnitHealthAlert )
				else --good health, set to default color
					W_Set_Unit_Button_Texture( unitButtonName, W_kUnitHealthOK )
				end
			end
			
		end
	else --offline so show them disconnected
		getglobal(unitButtonNameTextFrame):SetFontObject( W_kFontDisable );
	end
end

-- handle unit health events for player or someone in the players party (1-4)
function W_Handle_Party_Unit_Health_Event( unitID )
	--W_ShowMessge("Handle Party Unit Health Event for "..unitID):
	
	local unitButtonName, unitButtonNameTextFrame;
	local online, isDead;
	local partyMemberNum;
	
	if ( unitID == "player" ) then
		unitButtonName = "W_Unit_Button1";
		online = true;
	else
		partyMemberNum = W_GetPartyMemberNum( unitID );
		if ( partyMemberNum ) then -- nil if not a party member
			unitButtonName = "W_Unit_Button"..(partyMemberNum + 1) -- party1 would be button 2, party2 is button 3 etc
			online = UnitIsConnected( unitID );
		end
	end

	if ( unitButtonName ~= nil ) then
		--W_ShowMessage("  unitButtonName is "..unitButtonName);
		unitButtonNameTextFrame = (unitButtonName.."_UnitName");
		isDead = UnitIsDead( unitID );
		
		if ( online ) then
			if ( UnitIsUnit( unitID, "player") ) then --if they were offline and are now online, color the text correctly
				getglobal(unitButtonNameTextFrame):SetFontObject( W_kFontNamePlayer );
			else
				getglobal(unitButtonNameTextFrame):SetFontObject( W_kFontNameOthers );
			end
			
			if ( isDead ) then
				W_Set_Unit_Button_Texture( unitButtonName, W_kUnitDead ); --unit is dead, grey out the unit button texture
				W_Set_Health_Bar_Value( unitButtonName, 1); --set the healthbar to 1 (0 doesnt work)
			else --alive, update health bar
				local maxHealth = UnitHealthMax(unitID);
				local currentHealth = UnitHealth(unitID);
				local healthFraction = currentHealth / maxHealth;
				local healthBarAmount = W_kHealthBarWidth * healthFraction;
				
				W_Set_Unit_Button_Texture( unitButtonName, W_kUnitAlive);
				W_Set_Health_Bar_Value( unitButtonName, healthBarAmount );
				
				if W_gHealthLevel ~= "Off" then --if the user wants to be notified for low health, let them know
					--if the unit has low health, make the texture of the unit button red to get attention
					local healthTargetPercent = tonumber(W_gHealthLevel)/100; --health level is the % health selected in the drop down
					if ( healthFraction < healthTargetPercent ) then -- w_gHealthLevel is a saved var
						W_Set_Unit_Button_Texture( unitButtonName, W_kUnitHealthAlert )
					else --good health, set to default color
						W_Set_Unit_Button_Texture( unitButtonName, W_kUnitHealthOK )
					end
				end
				
			end
		else --offline so show them disconnected
			getglobal(unitButtonNameTextFrame):SetFontObject( W_kFontDisable );
		end
	end
end

function W_Set_Unit_Button_Texture( buttonName, whichTexture ) --set the texture of the entire unit button
	--W_ShowMessage("Set Unit Button Texture for button: "..buttonName);
	if ( buttonName ~= nil) then
		local r, g, b, a;
		
		if ( whichTexture == W_kUnitAlive ) then
			r = 1; g = 1; b = 1; a = .2;	
			
		elseif ( whichTexture == W_kUnitDead ) then
			r = .1; g = .1; b = .1; a = .8;
			
		elseif ( whichTexture == W_kUnitHealthAlert ) then
			r = 1; g = 0; b = 0; a = .5;
			
		elseif ( whichTexture == W_kUnitHealthOK ) then
			r = 1; g = 1; b = 1; a = .2;
			
		elseif ( whichTexture == W_kUnitDisconnected ) then
			r = 0; g = 0; b = 0; a = 0;	
			
		else
			r = 1; g = 1; b = 1; a = .2;
		end
		local buttonTextureName = buttonName.."_Texture";
		--W_ShowMessage("       buttonTextureName="..buttonTextureName)
		getglobal( buttonTextureName ):SetTexture( r, g, b, a );
	end
end

function W_Set_Health_Bar_Value( buttonName, healthBarValue ) --set the level of the health bar on the unit button
	local buttonHealthBarName = ( buttonName.."_HealthBar" ); --so we habve W_UnitButton1_HealthBar
	getglobal( buttonHealthBarName ):SetWidth( healthBarValue ); --adjust the width of the healthbar
end

function W_Set_Unit_Button_Curse_Texture( buttonName, isDebuffed) --set the texture of the curse area on the unit button
	local buttonCurseTextureName = ( buttonName.."_Curse_Texture" );

	if ( isDebuffed ) then
		getglobal( buttonCurseTextureName ):SetTexture( .97, 1, .38, 1 ); --yellow
	else
		getglobal( buttonCurseTextureName ):SetTexture( nil );
	end
end

--
-- Set up and Handle the Health Level drop down
--
function W_Health_DropDown_OnShow()
	--W_ShowMessage("W_Health_DropDown_OnLoad");
	
	local func = function()
		W_Health_DropDown_Initialize( this:GetParent() ); --define the init function for the health dropdown
	end
	
	UIDropDownMenu_Initialize( this, func ); --tell this drop down to init with the passed in function
	UIDropDownMenu_SetSelectedName( this, W_gHealthLevel ); --set the health level to the saved var
	UIDropDownMenu_SetWidth( 50, this ); --set the width of the dropdown
end

function W_Health_DropDown_Initialize( ddFrame )
	--W_ShowMessage("W_Health_DropDown_Initialize");
	
	local func = function()
		W_Health_DropDown_Handler( ddFrame );
	end
	
	local info = UIDropDownMenu_CreateInfo(); --set up the info table that will be going into the drop down
	
	for i = 0,90,10 do -- health levels are 0 to 90 in increments of 10
		if ( i == 0 ) then
			info.text = "Off";
		else
			info.text = i;
		end
		
		info.func = func;
		info.checked = ( i == W_gHealthLevel );
		UIDropDownMenu_AddButton(info); --add the button to the drop down
	end
end

function W_Health_DropDown_Handler( ddFrame )
	local selectedRow = this:GetID();
	UIDropDownMenu_SetSelectedID( ddFrame, this:GetID() ) --set the drop down to that row
	
	local healthLevel = this:GetText() -- get the current text
	
	W_gHealthLevel = healthLevel; --keep track of the new selection so it will be saved when we log
end

-- End Handle Health Drop Down Events


-- called when the player, a party or a raid member etc buffs/debuffs change.
function W_Handle_Unit_Aura_Event( unitID )
	--W_ShowMessage( "Unit Aura Event for unitid "..unitID);
	local debuffName, debuffRank, debuffTexture, debuffApplications, debuffType;
	local i = 1;
	local showDispellable = 1; -- 1 = only return debuffs that are dispellable, 0 = all buffs
	local foundOne, unitButtonName, unitName, partyMemberNum;
	local numRaidMembers = GetNumRaidMembers();
	local partyMemberNum;
	
	if ( numRaidMembers == 0 ) then -- we are not in a raid
		if ( unitID == "player" ) then -- player is button 1 when by themselves or in a party
			unitButtonName = "W_Unit_Button1";
		else -- unit was not player, so either a party member or a party member pet (which we ignore)
			partyMemberNum = W_GetPartyMemberNum( unitID );
			if ( partyMemberNum ) then -- nil if not a party member
				unitButtonName = "W_Unit_Button"..(partyMemberNum + 1) -- party1 would be button 2, party2 is button 3 etc
			end
		end
	else -- we are in a raid
		unitButtonName = W_Get_Unit_Button_From_UnitID( unitID ); --look up which button this raid unit is in via the global table
	end
	
	if ( unitButtonName ~= nil) then
		while ( UnitDebuff( unitID, i) ) do -- while the unit has debuffs, walk through them as see if there are any we can fix
			debuffName, debuffRank, debuffTexture, debuffApplications, debuffType = UnitDebuff( unitID, i, showDispellable );
			if (debuffTexture ~= nil) then
				foundOne = true;
				break; -- found one so exit the loop
			end
			i=i+1;
		end
		if (foundOne) then -- we found at least 1 debuff we can dispell - highlight the units button
			W_Set_Unit_Button_Curse_Texture( unitButtonName, true);
		else -- nope, no debuffs, set the unit button back to normal
			W_Set_Unit_Button_Curse_Texture( unitButtonName, false);
		end
	end
end

-- called when we enter or leave combat. Enable or disable the drop downs depending on combat status
function W_Handle_Combat( isInCombat )
	if ( isInCombat ) then
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			local dropDownName = "W_DropDown"..i
			UIDropDownMenu_DisableDropDown( getglobal( dropDownName ) );
		end
	else
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			local dropDownName = "W_DropDown"..i
			UIDropDownMenu_EnableDropDown( getglobal( dropDownName ) );
		end
	end
end
--
--------  END EVENT HANDLERS --------
--


--
-- functions called to set up the unit buttons to respond to mouse clicks
--
-- called from the handle raid roster update and party change events to assign units to unit buttons
function W_Set_Unit_Button_Name( buttonName, unitID)

	local buttonNameUnit = (buttonName.."_UnitName"); -- get the entire string for the text part of the button name, like W_UnitButton1_UnitName
	local unitName = UnitName(unitID); -- unit ID will be "player" or "party1" or "raid1", and this will give us the actual name
	
	getglobal(buttonNameUnit):SetText(unitName); --set the text of the passed in button to the unit name
	getglobal(buttonName):SetAttribute("unit", unitID); --set the function of the button to the passed in unit
	
	if ( UnitIsUnit( unitID, "player" ) ) then -- if this button is the player button, make it stand out so it is more easily picked out
		getglobal(buttonNameUnit):SetFontObject( W_kFontNamePlayer );
	else
		if ( UnitIsConnected( unitID ) ) then --if the unit is connected, standard font color
			getglobal(buttonNameUnit):SetFontObject( W_kFontNameOthers );
		else --if the unit disconnects then change the font color to disconnected color
			getglobal(buttonNameUnit):SetFontObject( W_kFontNameDisable );
		end
	end
	
	--W_ShowMessage( getglobal(buttonName):GetAttribute("w_unit_name") );
	
end

--called from handle raid roster update, dropdown selection change events and default button click.
--      Assign mouse clicks to the passed in unit button
function W_Set_Unit_Button_Click( unitButtonName )
	--W_ShowMessage("setting ".. unitButtonName);
	W_Set_Click("", "1", unitButtonName, W_SavedSpells[ "W_DropDown1"] ); --Left Click is DropDown1
	W_Set_Click("", "2", unitButtonName, W_SavedSpells[ "W_DropDown3"]); -- Right Click is DropDown3
	W_Set_Click("", "3", unitButtonName, W_SavedSpells[ "W_DropDown2"]); --Middle Button Click is DropDown2
	W_Set_Click("ctrl", "1", unitButtonName, W_SavedSpells[ "W_DropDown4"]); --Ctrl Left
	W_Set_Click("ctrl", "2", unitButtonName, W_SavedSpells[ "W_DropDown6"]); --Ctrl Right
	W_Set_Click("ctrl", "3", unitButtonName, W_SavedSpells[ "W_DropDown5"]); --Ctrl Middle
	W_Set_Click("shift", "1", unitButtonName, W_SavedSpells[ "W_DropDown7"]); --Shift Left
	W_Set_Click("shift", "2", unitButtonName, W_SavedSpells[ "W_DropDown9"]); --Shift Right
	W_Set_Click("shift", "3", unitButtonName, W_SavedSpells[ "W_DropDown8"]); --Shift Middle
	W_Set_Click("alt", "1", unitButtonName, W_SavedSpells[ "W_DropDown10"]); --Alt Left
	W_Set_Click("alt", "2", unitButtonName, W_SavedSpells[ "W_DropDown12"]); --Alt Right
	W_Set_Click("alt", "3", unitButtonName, W_SavedSpells[ "W_DropDown11"]); --Alt Middle
end

-- Called from W_Set_Unit_Button_Click to set what the mouse click does in reponse to the players frameButton
function W_Set_Click(keyPress, mouseButton, unitButtonName, whichSpell)
	--keyPress will be filled if there is a keyboard button pressed along with the mouse click
	--mouseButton is: 1=Left, 2=Right, 3=Middle, 4 and 5
	--frameButtonName is the frame button on the screen we are assigning the clicks to
	--whichSpell is the name of the spell
	
	local myUnitID, myUnitName, myUnitButton;
	
	if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
		keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
	end
	
	myUnitButton = getglobal( unitButtonName ); --get a pointer to the unitButton
	
	if ( myUnitButton:GetAttribute("unit") ~= nil ) then -- the unit attrib would be nil if it is does not have a unit assigned to it
		if ( whichSpell == "Trade" ) then --this button will be set to trade
			myUnitID = myUnitButton:GetAttribute("unit");
			myUnitButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Trade");
			myUnitButton.W_Perform_Trade = function() InitiateTrade(myUnitID); W_ShowMessage("Opening trade window"); end;
		elseif ( whichSpell == "Whisper") then
			myUnitID = myUnitButton:GetAttribute("unit");
			myUnitName = UnitName( myUnitID );
			local chatType = "WHISPER";
			local channel = GetChannelName( chatType );
			myUnitButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Chat_Message");
			myUnitButton.W_Perform_Chat_Message = function() ChatFrame_SendTell(myUnitName); W_ShowMessage("Whisper"); end;
		else --we are setting the button to a spell cast or action
			myUnitButton:SetAttribute( (keyPress.."type"..mouseButton), "spell");
			myUnitButton:SetAttribute( (keyPress.."spell"..mouseButton), whichSpell);
		end
	end
end

function W_Open_Trade_Macro()
	--TargetByName("Kaylanna");
	InitiateTrade("target");
	W_ShowMessage("opening trade window");
end

-- called from the handle raid roster update and party change events to clear the unused buttons
function W_ClearButton(whichUnitButtonName)
	--W_ShowMessage("Clearing Button "..whichUnitButtonName);
	
	local ub = getglobal( whichUnitButtonName );
	local ubTextFrame = getglobal( whichUnitButtonName.."_UnitName" );
	
	-- set the text of the button to -
	ubTextFrame:SetText("-");
	ubTextFrame:SetFontObject( W_kFontNameOthers );
	
	-- set the action of the button to nil
	ub:SetAttribute("type", "stop"); --set the function of the button to nothing
	ub:SetAttribute("unit", nil); --set the unit attrib (the unit name) to nothing
	
	-- set the color of the button to standard
	W_Set_Unit_Button_Texture( whichUnitButtonName, W_kUnitHealthOK );
	
	-- set the curse of the button to off
	W_Set_Unit_Button_Curse_Texture( whichUnitButtonName, false);
	
	-- set the health bar to full health
	W_Set_Health_Bar_Value( whichUnitButtonName, W_kHealthBarWidth);
end


--
-- W_DropDownMenu stuff: the onload function defines which function init's the button (which is automatically called)
--
-- load up the action buttons with default spells if there were no saved spells
function W_Set_Saved_Spells()
	W_ShowMessage("Setting spells to default");
	if ( W_SavedSpells == nil ) then
		W_SavedSpells = {}; --create the table
	end
	
	local class = UnitClass("player"); --what class of player so we know which table to get the defaults from
	local defaultTableName = "W_"..class.."DefaultSpellTable"; -- the default spell table for this class ie W_PriestDefaultSpellTable

	for i = 1,W_kMaxDropDownButtons do --for each drop down button, get what it's default should be from the default tables for this char type
		local dropDownButtonName = "W_DropDown"..i;
		W_SavedSpells[ dropDownButtonName ] = getglobal(defaultTableName)[i]; -- this is W_DropDown1="Greater Heal" and W_DropDown2="Renew" etc
		--W_ShowMessage( dropDownButtonName.." = "..getglobal(defaultTableName)[i] );
	end
	
	--W_ShowMessage( "Saved Spell Count = "..table.getn(W_SavedSpells) );
end

--	use for troubleshooting
function W_Show_Spells() -- this function is for troubleshooting
	for k, v in pairs(  W_SavedSpells ) do --iterate over the key, values pairs from the spell table appropriate for this char
		local value = W_SavedSpells[k];
	end
end

function W_DropDown_OnShow()
	--W_ShowMessage("W_DropDown_OnShow");
	
	local func = function()
		W_DropDown_Initialize( this:GetParent() );	-- define the init function for the drop down
	end

	UIDropDownMenu_Initialize( this, func ); --tell this drop down to init with the passed in function
	local spellName = W_SavedSpells[ this:GetName() ]; --get the spell to set this to
	UIDropDownMenu_SetSelectedName( this, spellName );
	UIDropDownMenu_SetWidth( 150, this );
end

function W_DropDown_Initialize( ddFrame )
	local index, spellName;
	
	local func = function()
		W_DropDown_Handler( ddFrame ); --define the function that handles drop down selections
	end

	local charType = UnitClass("player");
	local tableName = ("W_"..charType.."SpellTable"); --figure out which table to use based on char type
	local t = getglobal(tableName); -- a reference to the table
	local info = UIDropDownMenu_CreateInfo();
	local currentSelectedSpell = W_SavedSpells[ this:GetName() ]  --what was the previously selected spell

	for index, spellName in pairs( t ) do --iterate over the key, values pairs from the spell table for this char
		info.text =  spellName; --assign the spell name to the dropdown info
		info.value = index; --index will the spell number of each spell in the table
		info.func = func; --define the function that is called when an option in the drop down is selected
		info.checked = ( spellName == currentSelectedSpell );
		UIDropDownMenu_AddButton(info); --add the button to the drop down
	end

end

function W_DropDown_Handler( ddFrame )
	--W_ShowMessage("W_DropDown_OnClick");
	
	local selectedRow = this:GetID(); --which row did the user click
	UIDropDownMenu_SetSelectedID( ddFrame, this:GetID() ) --set the drop down to that row
	
	local spell = this:GetText(); --get the name of the selected spell
	local dropDownName = ddFrame:GetName(); --get the name of the dropdown
	W_SavedSpells[dropDownName] = spell; --store the newly selected spell with this dropdown in the saved spells table
	
	W_Update_Unit_Buttons(); --update the unit buttons to reflect this new spell
end

--walk through and update all of the unit buttons to reflect a change in spell or unit etc
function W_Update_Unit_Buttons() 
	for i = 1,W_kMaxUnitButtons do --now that we changed a spell, walk through and update all of the unit buttons to reflect the change
		W_Set_Unit_Button_Click( "W_Unit_Button"..i );
	end
end



--
-- MISC functions from here down
--

-- Display a message in the chat frame
function W_ShowMessage(aMessage)
	local r,g,b,a
	r = 0;
	g = 1;
	b = 0;
	a = .5;
	
	if ( aMessage ~= nil ) then
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: " ..aMessage, r, g, b, a);
	else
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: nil message");
	end
end

-- handle clicking the set defaults button - need to add an Are You Sure dialog
function W_Default_Button_Click() --clicking the button sets the spells back to their defaults
	--W_ShowMessage("Button click");
	
	W_Set_Saved_Spells(); --set the saved spell table to the default spells for this type of char (priest, mage etc)
	
	for i = 1,W_kMaxDropDownButtons do --now walk through the buttons and set their text
		local dropDownName = "W_DropDown"..i --each drop down
		local ddFrame = getglobal( dropDownName ); -- and the drop down frame
		local spellName = W_SavedSpells[dropDownName]; -- and the spells that we want to default the drop down to
		--W_ShowMessage( dropDownName.." = "..spellName );
		UIDropDownMenu_SetSelectedName( ddFrame, spellName); --set the selected item in the list
		UIDropDownMenu_SetText( spellName, ddFrame ); --also put the name of the spell in the text area
	end
	
	W_Update_Unit_Buttons();
	
	-- set the health value drop down to default
	UIDropDownMenu_SetSelectedName( W_Health_DropDown, "Off" ); --set the dropdown selection
	W_gHealthLevel = "Off"
end

function W_Help_Test_Button_Click()
	W_ShowMessage("help test button");
end

function W_Test_Button_Click()
	W_ShowMessage("test button");
end



function W_Info_Button_Click() -- show/hide the info frame when the user clicks the ? button
	local frame = WhammyInfoFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	end
end

function W_Info_Onload(self)
	--W_ShowMessage("info onload");
	local infoText
	WhammyInfoFrame:SetWidth( 400 ); --adjust the size of the frame
	WhammyInfoFrame:SetHeight( 300 );
	
	WhammyInfoFrame_Heading:SetText("Whammy "..WhammyVers);
	
	WhammyInfoFrame_Text:SetPoint("TOPLEFT", "WhammyInfoFrame", 10, -35);
	WhammyInfoFrame_Text:SetJustifyH("LEFT");
	
			   infoText ="Whammy enables you to quickly heal, decurse, and perform|n";
	infoText = infoText.."   other functions to yourself, party or raid members through|n";
	infoText = infoText.."   a simple point and click interface.|n";
	infoText = infoText.."|n";
	infoText = infoText.."|n";
	infoText = infoText.."/whammy             - shows/hides the Whammy window|n";
	infoText = infoText.."|n";
	infoText = infoText.."To drag Whammy - Left Click and Drag title bar|n";
	infoText = infoText.."|n";
	infoText = infoText.."Configure Clicks   - Right Click in title bar|n";
	infoText = infoText.."|n";
	infoText = infoText.."Use Health Alert to set the percentage health to notify you|n";
	infoText = infoText.."   when a unit needs healing. The Unit Button will turn Red|n";
	infoText = infoText.."   when their health drops below this point.|n";
	infoText = infoText.."|n"
	infoText = infoText.."Each Unit Button has:|n"
	infoText = infoText.."    a health bar at the bottom (in green)|n";
	infoText = infoText.."    a curse/ailment bar at the top (yellow when cursed)|n";
	infoText = infoText.."|n";
	infoText = infoText.."Your character text always appears in green to easily|n";
	infoText = infoText.."   distinguish it from other characters.|n";
	
	WhammyInfoFrame_Text:SetText(infoText);
end


-- called to adjust the whammy frame size depending on single player, party or raid size
function W_Set_Whammy_Frame_Size( aGroupType )
	local newWidth, newHeight, numToShow;
	
	--aGroupType = W_kRaid;

	if ( aGroupType == W_kPlayer ) then
		newWidth = 99; newHeight = 60; numToShow = 1; --60 and 1 is default size for single player
	elseif ( aGroupType == W_kParty ) then
		newWidth = 99; newHeight = 143; numToShow = 5;
	--elseif ( aGroupType == W_kRaid20 ) then
	--	newWidth = 318; newHeight = 160; numToShow = 20;
	elseif ( aGroupType == W_kRaid ) then
		newWidth = 318; newHeight = 254; numToShow = 40;
	else
		newWidth = 100; newHeight = 80; numToShow = 1;
	end
	
	for i = 1,numToShow do --show the unit buttons that need to be seen
		local buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Show();
	end
	
	for i = numToShow+1,40 do --hide the unused unit buttons
		local buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Hide();
	end
	
	WhammyFrame:SetWidth( newWidth ); --adjust the size of the frame
	WhammyFrame:SetHeight( newHeight );
end


-- create all of the potential unit buttons. They start off hidden except for #1, which by default is the player
function W_Create_Unit_Buttons()
	for i = 1,40 do
		W_Create_Button(i);
	end
end

function W_Create_Button( buttonNum )
	local row, col;
	local buttonFrameName = "W_Unit_Button"..buttonNum;
	local buttonWidth = 70;
	local buttonHeight = 18;
	local leftMargin = 15;
	local gutter = 3;
	local topMargin = 28;
	local additionalTopMargin = ( buttonHeight * 4 ) + ( gutter * 3 ) + topMargin;
	
	if ( buttonNum < 6 ) then
		col = 0; row = buttonNum - 1;
	elseif ( buttonNum > 5 and buttonNum < 11 ) then
		col = 1; row = buttonNum - 5 - 1;
	elseif (buttonNum > 10 and buttonNum < 16) then
		col = 2; row = buttonNum - 10 - 1;
	elseif (buttonNum > 15 and buttonNum < 21) then
		col = 3; row = buttonNum - 15 - 1;
		
	elseif (buttonNum > 20 and buttonNum < 26) then
		col = 0; row = buttonNum - 20 - 1;
	elseif (buttonNum > 25 and buttonNum < 31) then
		col = 1; row = buttonNum - 25 - 1;
	elseif (buttonNum > 30 and buttonNum < 36) then
		col = 2; row = buttonNum - 30 - 1;
	elseif (buttonNum > 35 and buttonNum < 41) then
		col = 3; row = buttonNum - 35 - 1;
	end
	
	local x = leftMargin + (buttonWidth * col) + (gutter * col);
	local y = topMargin + (buttonHeight * row) + (gutter * row);
	
	if ( buttonNum > 20 ) then --if this is members 20-40, move them down below the first 20
		y = y + additionalTopMargin;
	end
	
	--create the frame: pass the  type, name, child of, template
	local f = CreateFrame("Button", buttonFrameName, WhammyFrame, "WhammyUnitButtonTemplate");

	f:SetPoint("TOPLEFT", "WhammyFrame", "TOPLEFT", x, -y);
	f:SetID(buttonNum);
	f:Hide(); --start off hidden
end
-- End Unit button creation



function W_Is_Unit_Player_Party_Raid( unitID ) --returns if the userID is player, party or raid member
	-- ignore everyone except player, party or raid members, and also ignore it if it's target

	return (   (unitID == "player" or UnitInParty(unitID) or UnitInRaid(unitID))   and   userID ~= "target" );
end

-- toggle the config frame shown and hidden
function W_Config_Frame_Toggle()
	local configFrame = WhammyConfigFrame;
	local wFrame = WhammyFrame;
	local parentFrame = UIParent; --need to make sure we open the config frame in the work area
	
	if ( configFrame ) then
		if ( configFrame:IsVisible() ) then --hide the frame if it's visible
			configFrame:Hide();
		else --show the frame if it's hidden

			local parentFrameWidth = parentFrame:GetWidth(); -- get the width of the main window
			local configFrameWidth = configFrame:GetWidth();
			local wFrameLeft = floor( wFrame:GetLeft() + 0.5); --get left and make it in integer
			local wFrameTop = floor( wFrame:GetTop() + 0.5);
			local wFrameWidth = floor( wFrame:GetWidth() + 0.5);
			
			local totalWidth = wFrameLeft + wFrameWidth + configFrameWidth
			
			configFrame:ClearAllPoints(); --releases the frame so it can be repositioned
							
			if ( totalWidth > parentFrameWidth ) then --if the config window is off the right side of the screen, open on the left of Whammy
				--obj:SetPoint(point, frame, relativePoint , x, y);
				configFrame:SetPoint("TOPRIGHT", wFrame, "TOPLEFT", -5, -5)
			else -- it was not off the right side of the screen, so open it on the right
				configFrame:SetPoint("TOPLEFT", wFrame, "TOPRIGHT", 5, -5)
			end
			
			configFrame:Show();
			
		end
	end
end

-- handle our one and only slash command
function W_Slash_Command_Handler(msg)
	--W_ShowMessage("slash handler");
	local frame = WhammyFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
			if ( WhammyConfigFrame:IsVisible() ) then
				WhammyConfigFrame:Hide();
			end
		else
			frame:Show();
		end
	end
end

--get the unit name from the button passed in
function W_Get_Unit_Name_From_Button( aButton )
	local unitButtonName = aButton.."_UnitName"
	local unitName = getglobal(unitButtonName):GetText();
	
	return unitName;
end

-- get a party member number from the unitID - return nil if a pet
function W_GetPartyMemberNum( unitID )
	local id, numberFound;
	
	if ( string.find( unitID, "partypet") == nil ) then --ignore party pets for now
		id, numberFound = gsub( unitID, "party([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		
		if ( numberFound > 0 ) then
			return id;
		else
			return nil;
		end
	else -- it was a pet, return nil
		return nil
	end
end

-- for a raid: return the unit button the unit is in
function W_Get_Unit_Button_From_UnitID( unitID ) --this could be passed raid1-40
	local buttonName = W_gUnitButtonTable[ unitID ]; --this table is filled in whenever there is a raid roster update
	
	return buttonName
end